from django.contrib import admin

from .forms import TemplateForm
from .models import Template


class TemplateAdmin(admin.ModelAdmin):
    form = TemplateForm
    list_display = ('name', 'format', 'linked_object', 'added', 'updated')


admin.site.register(Template, TemplateAdmin)



from django.db import models
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType


FORMAT_CHOICES = (
    ('odt', 'odt'),
    ('html', 'html'),
    ('docx', 'docx'),
)


class Template(models.Model):
    name = models.CharField(max_length=256, unique=True)
    format = models.CharField(max_length=256, choices=FORMAT_CHOICES)
    content = models.TextField()

    content_type = models.ForeignKey(ContentType, on_delete=models.PROTECT)
    object_id = models.PositiveIntegerField()
    linked_object = GenericForeignKey()

    added = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)



from rest_framework.serializers import FileField, ModelSerializer

from .models import Template
from .utils import FmtMimeMapping, from_bytes_to_str


class TemplateSerializer(ModelSerializer):
    content = FileField(write_only=True)

    class Meta:
        model = Template
        fields = '__all__'
        extra_kwargs = {
            'added': {'read_only': True},
            'updated': {'read_only': True},
        }

    def validate_content(self, data):
        fmt = FmtMimeMapping(data.content_type).name
        b_content = data.file.read()
        return from_bytes_to_str(b_content, fmt)



from django.forms import FileField, ModelForm

from .models import Template
from .utils import from_bytes_to_str


class TemplateForm(ModelForm):
    content = FileField()

    class Meta:
        model = Template
        fields = ['name', 'format', 'content', 'content_type', 'object_id']

    def clean_content(self):
        b_content = self.cleaned_data['content'].file.read()
        fmt = self.cleaned_data['format']
        return from_bytes_to_str(b_content, fmt)



# Generated by Django 2.2.2 on 2019-08-05 15:38

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='Template',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=256, unique=True)),
                ('format', models.CharField(choices=[('odt', 'odt'), ('html', 'html'), ('docx', 'docx')], max_length=256)),
                ('content', models.TextField()),
                ('object_id', models.PositiveIntegerField()),
                ('added', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='contenttypes.ContentType')),
            ],
        ),
    ]



from base64 import b64decode, b64encode
from enum import Enum


class FmtMimeMapping(Enum):
    docx = 'application/vnd.openxmlformats-officedocument.wordprocessingml.document'
    odt = 'application/vnd.oasis.opendocument.text'
    html = 'text/html'


def from_bytes_to_str(b_obj, fmt):
    if fmt == 'html':
        return b_obj.decode()
    elif fmt == 'odt' or fmt == 'docx':
        return b64encode(b_obj).decode()
    else:
        return None


def from_str_to_bytes(str_obj, fmt):
    if fmt == 'odt' or fmt == 'docx':
        return b64decode(str_obj.encode())
    elif fmt == 'html':
        return str_obj.encode()
    else:
        return None



from django.urls import include, path
from rest_framework import routers

from .views import TemplateViewSet

router = routers.SimpleRouter()
router.register(r'document-template', TemplateViewSet)

urlpatterns = [
    path('', include(router.urls)),
]



from django.http import HttpResponse
from rest_framework.viewsets import ModelViewSet
from rest_framework.decorators import action

from .models import Template
from .serializers import TemplateSerializer
from .utils import FmtMimeMapping, from_str_to_bytes


class TemplateViewSet(ModelViewSet):
    queryset = Template.objects.all()
    serializer_class = TemplateSerializer

    @action(detail=True, methods=['get'], url_path='content')
    def content(self, request, *args, **kwargs):
        obj = self.get_object()
        return HttpResponse(
            content=from_str_to_bytes(obj.content, obj.format),
            content_type=getattr(FmtMimeMapping, obj.format).value
        )



#!/usr/bin/env python

import os
from setuptools import setup, find_packages

HERE = os.path.abspath(os.path.dirname(__file__))

README = open(os.path.join(HERE, 'README.md')).read()
CHANGES = open(os.path.join(HERE, 'CHANGES.md')).read()


setup(
    name='django-template-model',
    version=open(os.path.join(HERE, 'template_model', 'VERSION.md')).read().strip(),
    include_package_data=True,
    author="Makina Corpus",
    author_email="terralego-pypi@makina-corpus.com",
    description='Template model.',
    long_description=README + '\n\n' + CHANGES,
    description_content_type="text/markdown",
    long_description_content_type="text/markdown",
    packages=find_packages(),
    url='https://github.com/courtem/django-template-model',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    install_requires=['Django>=2.1.0,<3.0.0', 'djangorestframework>=3.10.0,<3.11.0'],
)
